@@ -23,9 +23,6 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libsexy/sexy-tooltip.h>
 
 #include "conversation-panel.h"
 #include "gui.h"
@@ -44,51 +41,58 @@
 
 typedef struct _fe_lastlog_info fe_lastlog_info;
 
-static void     conversation_panel_class_init         (ConversationPanelClass *klass);
-static void     conversation_panel_init               (ConversationPanel      *panel);
-static void     conversation_panel_finalize           (GObject                *object);
-static void     conversation_panel_realize            (GtkWidget              *widget);
-static int      conversation_panel_check_word         (GtkWidget              *xtext,
-                                                       char                   *word,
-                                                       int                     len);
-static void     conversation_panel_clicked_word       (GtkWidget              *xtext,
-                                                       char                   *word,
-                                                       GdkEventButton         *event,
-                                                       ConversationPanel      *panel);
-static void     conversation_panel_enter_word         (GtkWidget              *xtext,
-                                                       char                   *word,
-                                                       ConversationPanel      *panel);
-static void     conversation_panel_leave_word         (GtkWidget              *xtext,
-                                                       char                   *word,
-                                                       ConversationPanel      *panel);
-static gboolean conversation_panel_lost_focus         (GtkWidget              *widget,
-                                                       GdkEventFocus          *event,
-                                                       ConversationPanel      *panel);
-static void     conversation_panel_set_font           (ConversationPanel      *panel);
-static void     conversation_panel_font_changed       (GConfClient            *client,
-                                                       guint                   cnxn_id,
-                                                       GConfEntry             *entry,
-                                                       ConversationPanel      *panel);
+static void conversation_panel_class_init(ConversationPanelClass *klass);
+static void conversation_panel_init(ConversationPanel *panel);
+static void conversation_panel_finalize(GObject *object);
+static void conversation_panel_realize(GtkWidget *widget);
+static int conversation_panel_check_word(GtkWidget *xtext,
+                                         char *word,
+                                         int len);
+static void conversation_panel_clicked_word(GtkWidget *xtext,
+                                            char *word,
+                                            GdkEventButton *event,
+                                            ConversationPanel *panel);
+static void conversation_panel_enter_word(GtkWidget *xtext,
+                                          char *word,
+                                          ConversationPanel *panel);
+static void conversation_panel_leave_word(GtkWidget *xtext,
+                                          char *word,
+                                          ConversationPanel *panel);
+static gboolean conversation_panel_lost_focus(GtkWidget *widget,
+                                              GdkEventFocus *event,
+                                              ConversationPanel *panel);
+static void conversation_panel_set_font(ConversationPanel *panel);
+static void conversation_panel_font_changed(GConfClient *client,
+                                            guint cnxn_id,
+                                            GConfEntry *entry,
+                                            ConversationPanel *panel);
 static void     conversation_panel_set_background     (ConversationPanel      *panel);
 static void     conversation_panel_background_changed (GConfClient            *client,
                                                        guint                   cnxn_id,
                                                        GConfEntry             *entry,
                                                        ConversationPanel      *panel);
-static void     timestamps_changed                    (GConfClient            *client,
-                                                       guint                   cnxn_id,
-                                                       GConfEntry             *entry,
-                                                       xtext_buffer           *buffer);
-static void     redundant_nickstamp_changed           (GConfClient            *client,
-                                                       guint                   cnxn_id,
-                                                       GConfEntry             *entry,
-                                                       ConversationPanel      *panel);
+static gboolean conversation_panel_query_tooltip      (GtkWidget  *widget,
+                                                       gint        x,
+                                                       gint        y,
+                                                       gboolean    keyboard_tooltip,
+                                                       GtkTooltip *tooltip);
+
+static void     timestamps_changed                    (GConfClient  *client,
+                                                       guint         cnxn_id,
+                                                       GConfEntry   *entry,
+                                                       xtext_buffer *buffer);
+
+static void     redundant_nickstamp_changed           (GConfClient       *client,
+                                                       guint              cnxn_id,
+                                                       GConfEntry        *entry,
+                                                       ConversationPanel *panel);
 static void     conversation_panel_print_line         (ConversationPanel      *panel,
                                                        xtext_buffer           *buffer,
-                                                       guchar                  *text,
+                                                       char                   *text,
                                                        int                     len,
                                                        gboolean                indent);
-static void     conversation_panel_lastlog_foreach    (XText                  *xtext,
-                                                       guchar                 *text,
+static void     conversation_panel_lastlog_foreach    (GtkXText               *xtext,
+                                                       char                   *text,
                                                        fe_lastlog_info        *info);
 
 static void     open_url                              (GtkAction              *action,
@@ -153,18 +157,19 @@ struct _ConversationPanelPriv
 	gchar          *selected_word;
 	GSList         *dropped_files;
 
-	GtkWidget      *current_tooltip;
-	guint           tooltip_timeout;
+	gchar          *tooltip_nick;
 	gboolean        redundant_nickstamps;
 };
 
+#define STOCK_MAIL_SEND "mail-send"
+
 static GtkActionEntry url_actions[] = {
 	{ "TextURLOpen", GTK_STOCK_OPEN, N_("_Open Link in Browser"), NULL, NULL, G_CALLBACK (open_url) },
 	{ "TextURLCopy", GTK_STOCK_COPY, N_("_Copy Link Location"),   NULL, NULL, G_CALLBACK (copy_text) },
 };
 
 static GtkActionEntry email_actions[] = {
-	{ "TextEmailSend", GNOME_STOCK_MAIL, N_("Se_nd Message To..."), NULL, NULL, G_CALLBACK (send_email) },
+	{ "TextEmailSend", STOCK_MAIL_SEND, N_("Se_nd Message To..."), NULL, NULL, G_CALLBACK (send_email) },
         { "TextEmailCopy", GTK_STOCK_COPY,   N_("_Copy Address"),       NULL, NULL, G_CALLBACK (copy_text) },
 };
 
@@ -180,24 +185,6 @@ static GtkActionEntry default_actions[] = {
 };
 
 #define DROP_FILE_PASTE_MAX_SIZE 1024
-enum
-{
-	TARGET_URI_LIST,
-	TARGET_UTF8_STRING,
-	TARGET_TEXT,
-	TARGET_COMPOUND_TEXT,
-	TARGET_STRING,
-	TARGET_TEXT_PLAIN,
-};
-
-static GtkTargetEntry target_table[] = {
-	{ "text/uri-list",  0, TARGET_URI_LIST },
-	{ "UTF8_STRING",    0, TARGET_UTF8_STRING },
-	{ "COMPOUND_TEXT",  0, TARGET_COMPOUND_TEXT },
-	{ "TEXT",           0, TARGET_TEXT },
-	{ "STRING",         0, TARGET_STRING },
-	{ "text/plain",     0, TARGET_TEXT_PLAIN }
-};
 
 static struct User dialog_user;
 
@@ -217,6 +204,7 @@ conversation_panel_class_init (ConversationPanelClass *klass)
 
 	widget_class = GTK_WIDGET_CLASS (klass);
 	widget_class->realize = conversation_panel_realize;
+	widget_class->query_tooltip = conversation_panel_query_tooltip;
 }
 
 static void
@@ -227,11 +215,11 @@ conversation_panel_init (ConversationPanel *panel)
 	GConfClient       *client;
 
 	panel->priv = g_new0 (ConversationPanelPriv, 1);
-	panel->priv->xtext     = xtext_new ();
-	panel->priv->scrollbar = gtk_vscrollbar_new (xtext_get_adjustment (XTEXT (panel->priv->xtext)));
+	panel->priv->xtext     = gtk_xtext_new (colors, prefs.indent_nicks);
+	panel->priv->scrollbar = gtk_vscrollbar_new (GTK_XTEXT (panel->priv->xtext)->adj);
 	frame                  = gtk_frame_new (NULL);
 
-	panel->priv->buffers            = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) xtext_buffer_free);
+	panel->priv->buffers            = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) gtk_xtext_buffer_free);
 	panel->priv->timestamp_notifies = g_hash_table_new      (g_direct_hash, g_direct_equal);
 
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
@@ -255,8 +243,8 @@ conversation_panel_init (ConversationPanel *panel)
 	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
 	g_object_unref (action_group);
 
-	panel->priv->tooltip_timeout = 0;
-	panel->priv->current_tooltip = NULL;
+	panel->priv->tooltip_nick = NULL;
+	gtk_widget_set_has_tooltip(GTK_WIDGET(panel), TRUE);
 
 	client = gconf_client_get_default ();
 	panel->priv->redundant_nickstamps =
@@ -266,7 +254,9 @@ conversation_panel_init (ConversationPanel *panel)
 	g_signal_connect (G_OBJECT (panel), "style-set", G_CALLBACK (style_set_callback), panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "drag_data_received", G_CALLBACK (drag_data_received), panel);
 	gtk_drag_dest_set (panel->priv->xtext, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
-	                   target_table, G_N_ELEMENTS (target_table), GDK_ACTION_COPY | GDK_ACTION_ASK);
+	                   NULL, 0, GDK_ACTION_COPY | GDK_ACTION_ASK);
+        gtk_drag_dest_add_uri_targets (panel->priv->xtext);
+        gtk_drag_dest_add_text_targets (panel->priv->xtext);
 
 #ifdef USE_XLIB
 	gdk_window_set_events (gdk_get_default_root_window (), GDK_PROPERTY_CHANGE_MASK);
@@ -306,14 +296,14 @@ conversation_panel_realize (GtkWidget *widget)
 	client = gconf_client_get_default ();
 
 	palette_alloc (panel->priv->xtext);
-	xtext_set_palette           (XTEXT (panel->priv->xtext), colors);
-	xtext_set_max_lines         (XTEXT (panel->priv->xtext), 3000);
-	xtext_set_indent            (XTEXT (panel->priv->xtext), prefs.indent_nicks);
-	xtext_set_max_indent        (XTEXT (panel->priv->xtext), prefs.max_auto_indent);
-	xtext_set_urlcheck_function (XTEXT (panel->priv->xtext), conversation_panel_check_word);
+	gtk_xtext_set_palette(GTK_XTEXT(panel->priv->xtext), colors);
+	gtk_xtext_set_max_lines(GTK_XTEXT(panel->priv->xtext), 3000);
+	gtk_xtext_set_indent(GTK_XTEXT(panel->priv->xtext), prefs.indent_nicks);
+	gtk_xtext_set_max_indent(GTK_XTEXT(panel->priv->xtext), prefs.max_auto_indent);
+	gtk_xtext_set_urlcheck_function(GTK_XTEXT(panel->priv->xtext), conversation_panel_check_word);
 
-	conversation_panel_set_font       (panel);
-	conversation_panel_set_background (panel);
+	conversation_panel_set_font(panel);
+	conversation_panel_set_background(panel);
 
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_click",      G_CALLBACK (conversation_panel_clicked_word), panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_enter",      G_CALLBACK (conversation_panel_enter_word),   panel);
@@ -492,83 +482,43 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 	}
 }
 
-static gboolean
-show_user_tooltip (gchar *nick)
-{
-	GtkWidget *tooltip;
-	ConversationPanel *panel;
-	struct User *user = NULL;
-	GdkRectangle rect;
-	GdkScreen *screen;
-	GtkWidget *vbox;
-	GdkDisplay *display;
-
-	panel = CONVERSATION_PANEL(gui.conversation_panel);
-
-	user = userlist_find (gui.current_session, nick);
-
-	if (user) {
-		tooltip = sexy_tooltip_new ();
-
-		display = gdk_display_get_default ();
-		gdk_display_get_pointer (display, &screen, &rect.x, &rect.y, NULL);
-		rect.y += 10;
-		rect.width = 1;
-		rect.height = 1;
-
-		vbox = get_user_vbox_infos (user);
-		gtk_container_add (GTK_CONTAINER (tooltip), vbox);
-		sexy_tooltip_position_to_rect (SEXY_TOOLTIP (tooltip), &rect, screen);
-		gtk_widget_show (tooltip);
-
-		panel->priv->current_tooltip = tooltip;
-	}
-
-	return FALSE;
-}
 
 static void
-conversation_panel_enter_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
+conversation_panel_enter_word(GtkWidget *xtext,
+                              char *word,
+                              ConversationPanel *panel)
+{
+	switch (conversation_panel_check_word(xtext, word, strlen (word))) {
+	case WORD_NICK:
+		panel->priv->tooltip_nick = g_strdup(word);
+		break;
+	}
+}
+
+
+static void
+conversation_panel_leave_word(GtkWidget *xtext,
+                              char *word,
+                              ConversationPanel *panel)
 {
 	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
 	case WORD_NICK:
-		{
-			panel->priv->tooltip_timeout = g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
-			break;
-		}
+		gtk_widget_set_has_tooltip(GTK_WIDGET(panel), FALSE);
+		gtk_widget_set_has_tooltip(GTK_WIDGET(panel), TRUE);
+		g_free(panel->priv->tooltip_nick);
+		panel->priv->tooltip_nick = NULL;
+		break;
 	}
 }
 
-static void
-conversation_panel_remove_tooltip (ConversationPanel *panel)
-{
-	if (panel->priv->tooltip_timeout) {
-		g_source_remove (panel->priv->tooltip_timeout);
-		panel->priv->tooltip_timeout = 0;
-	}
-
-	if (panel->priv->current_tooltip) {
-		gtk_widget_destroy (panel->priv->current_tooltip);
-		panel->priv->current_tooltip = NULL;
-	}
-}
-
-static void
-conversation_panel_leave_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
-{
-	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
-	case WORD_NICK:
-		{
-			conversation_panel_remove_tooltip (panel);
-			break;
-		}
-	}
-}
 
 static gboolean
-conversation_panel_lost_focus (GtkWidget *widget, GdkEventFocus *event, ConversationPanel *panel)
+conversation_panel_lost_focus(GtkWidget *widget,
+                              GdkEventFocus *event,
+                              ConversationPanel *panel)
 {
-	conversation_panel_remove_tooltip (panel);
+	g_free(panel->priv->tooltip_nick);
+	panel->priv->tooltip_nick = NULL;
 	return FALSE;
 }
 
@@ -594,7 +544,7 @@ conversation_panel_set_font (ConversationPanel *panel)
 	if (font == NULL)
 		font = g_strdup ("fixed 11");
 
-	xtext_set_font (XTEXT (panel->priv->xtext), font);
+	gtk_xtext_set_font (GTK_XTEXT (panel->priv->xtext), font);
 
 	g_free (font);
 }
@@ -606,9 +556,9 @@ conversation_panel_font_changed (GConfClient *client, guint cnxn_id, GConfEntry
 
 	conversation_panel_set_font (panel);
 
-	adj = xtext_get_adjustment (XTEXT (panel->priv->xtext));
+	adj = GTK_XTEXT (panel->priv->xtext)->adj;
 	gtk_adjustment_set_value (adj, adj->upper - adj->page_size);
-	xtext_refresh (XTEXT (panel->priv->xtext), FALSE);
+	gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), FALSE);
 }
 
 static void
@@ -620,12 +570,12 @@ conversation_panel_set_background (ConversationPanel *panel)
 	client = gconf_client_get_default ();
 	background_type = gconf_client_get_int (client, "/apps/xchat/main_window/background_type", NULL);
 	if (background_type == 0) {
-		xtext_set_tint (XTEXT (panel->priv->xtext), 0);
-		xtext_set_background (XTEXT (panel->priv->xtext), NULL, FALSE);
+		gtk_xtext_set_tint (GTK_XTEXT (panel->priv->xtext), 0, 0, 0);
+		gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), NULL, FALSE);
 	} else if (background_type == 1) {
 		gchar *filename = gconf_client_get_string (client, "/apps/xchat/main_window/background_image", NULL);
-		xtext_set_tint (XTEXT (panel->priv->xtext), 0);
-		xtext_set_background (XTEXT (panel->priv->xtext), NULL, FALSE);
+		gtk_xtext_set_tint (GTK_XTEXT (panel->priv->xtext), 0, 0, 0);
+		gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), NULL, FALSE);
 		if (filename) {
 			GdkPixbuf *pixbuf;
 			pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
@@ -639,7 +589,7 @@ conversation_panel_set_background (ConversationPanel *panel)
 				image = gdk_pixmap_new (NULL, width, height, 24);
 				gdk_draw_pixbuf (image, NULL, pixbuf, 0, 0, 0, 0, width, height, GDK_RGB_DITHER_NONE, 0, 0);
 
-				xtext_set_background (XTEXT (panel->priv->xtext), image, FALSE);
+				gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), image, FALSE);
 				g_object_unref (pixbuf);
 				g_object_unref (image);
 			}
@@ -648,8 +598,8 @@ conversation_panel_set_background (ConversationPanel *panel)
 	} else {
 		float transparency = gconf_client_get_float (client, "/apps/xchat/main_window/background_transparency", NULL);
 		int value = 255 - ((int) (transparency * 255));
-		xtext_set_tint (XTEXT (panel->priv->xtext), value);
-		xtext_set_background (XTEXT (panel->priv->xtext), NULL, TRUE);
+		gtk_xtext_set_tint (GTK_XTEXT (panel->priv->xtext), value, value, value);
+		gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), NULL, TRUE);
 	}
 	g_object_unref (client);
 }
@@ -658,13 +608,37 @@ static void
 conversation_panel_background_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, ConversationPanel *panel)
 {
 	conversation_panel_set_background (panel);
-	xtext_refresh (XTEXT (panel->priv->xtext), TRUE);
+	gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), TRUE);
 }
 
+
+gboolean
+conversation_panel_query_tooltip(GtkWidget *widget,
+                                 gint x,
+                                 gint y,
+                                 gboolean keyboard_tooltip,
+                                 GtkTooltip *tooltip)
+{
+	ConversationPanel *panel = CONVERSATION_PANEL (widget);
+
+	// FIXME: keyboard tooltips
+	if (panel->priv->tooltip_nick) {
+		struct User *user = userlist_find(gui.current_session,
+		                                  panel->priv->tooltip_nick);
+		if (user != NULL) {
+			gtk_tooltip_set_custom(tooltip,
+			                       get_user_vbox_infos(user));
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+
 static void
 timestamps_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, xtext_buffer *buffer)
 {
-	xtext_set_time_stamp (buffer, gconf_client_get_bool (client, entry->key, NULL));
+	gtk_xtext_set_time_stamp (buffer, gconf_client_get_bool (client, entry->key, NULL));
 }
 
 static void
@@ -676,8 +650,13 @@ redundant_nickstamp_changed (GConfClient *client, guint cnxn_id, GConfEntry *ent
 static void
 open_url (GtkAction *action, ConversationPanel *panel)
 {
+        ConversationPanelPriv *priv = panel->priv;
+
+        if (!priv->selected_word || !priv->selected_word[0])
+                return;
+
 	char *command;
-	command = g_strdup_printf ("URL %s", panel->priv->selected_word);
+	command = g_strdup_printf ("URL %s", priv->selected_word);
 	handle_command (gui.current_session, command, 1);
 	g_free (command);
 }
@@ -761,26 +740,7 @@ static void
 drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
                     GtkSelectionData *selection_data, guint info, guint time, ConversationPanel *panel)
 {
-	switch (info) {
-	case TARGET_TEXT:
-	case TARGET_STRING:
-	case TARGET_COMPOUND_TEXT:
-	case TARGET_UTF8_STRING:
-	case TARGET_TEXT_PLAIN:
-	{
-		guchar *txt;
-
-		txt = gtk_selection_data_get_text (selection_data);
-		if (gui.current_session != NULL) {
-			handle_multiline (gui.current_session, txt, TRUE, FALSE);
-		}
-
-		g_free (txt);
-		break;
-	}
-	case TARGET_URI_LIST:
-	{
-		guchar *uri_list;
+        if (gtk_targets_include_uri (&selection_data->target, 1)) {
 		gchar **uris;
 		gint nb_uri;
 
@@ -789,15 +749,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 			return;
 		}
 
-		if (selection_data->format != 8 || selection_data->length == 0) {
-			g_printerr (_("URI list dropped on XChat-GNOME had wrong format (%d) or length (%d)\n"),
-			            selection_data->format, selection_data->length);
-			return;
-		}
-
-		uri_list = g_strndup (selection_data->data, selection_data->length);
-		uris = g_strsplit (uri_list, "\r\n", 0);
-		g_free (uri_list);
+                uris = gtk_selection_data_get_uris (selection_data);
+                if (!uris)
+                        return;
 
 		free_dropped_files (panel);
 
@@ -850,7 +804,15 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 				}
 			}
 		}
-	}
+	} else if (gtk_targets_include_text (&selection_data->target, 1)) {
+		char *txt;
+
+                txt = (char *) gtk_selection_data_get_text (selection_data);
+		if (gui.current_session != NULL) {
+			handle_multiline (gui.current_session, txt, TRUE, FALSE);
+		}
+
+		g_free (txt);
 	}
 }
 
@@ -875,20 +837,23 @@ drop_send (ConversationPanel *panel)
 static void
 drop_paste (ConversationPanel *panel)
 {
-	GnomeVFSResult result;
-	gchar *contents, *uri;
+	gboolean res;
+	char *contents;
+	const char *uri = panel->priv->dropped_files->data;
+	GError *error = NULL;
 
 	g_return_if_fail (g_slist_length (panel->priv->dropped_files) == 1);
 
-	uri = panel->priv->dropped_files->data;
-	result = gnome_vfs_read_entire_file (uri, NULL, &contents);
-	if (result == GNOME_VFS_OK) {
+	res = g_file_get_contents (uri, &contents, NULL, NULL);
+
+	if (res) {
 		if (panel->priv->current != NULL) {
-			handle_multiline (panel->priv->current, (char *) contents, TRUE, FALSE);
+			handle_multiline (panel->priv->current, contents, TRUE, FALSE);
 		}
 		g_free (contents);
 	} else {
-		g_printerr (_("Error reading file \"%s\": %s\n"), uri, gnome_vfs_result_to_string (result));
+		g_printerr (_("Error reading file \"%s\": %s\n"), uri, error->message);
+		g_error_free (error);
 	}
 
 	free_dropped_files (panel);
@@ -920,13 +885,21 @@ on_default_copy_activate (GtkAction *action, ConversationPanel *panel)
 static gboolean
 uri_is_text (gchar *uri)
 {
-	gchar *mime;
+	GFile *file;
+	GFileInfo *info;
 	gboolean is_text = FALSE;
 
-	mime = gnome_vfs_get_mime_type (uri);
-	if (mime) {
-		is_text = g_str_has_prefix (mime, "text/");
-		g_free (mime);
+	file = g_file_new_for_uri (uri);
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+				  0, NULL, NULL);
+	g_object_unref (file);
+	
+	if (info) {
+		const char *type;
+		type = g_file_info_get_content_type (info);
+		is_text = g_content_type_is_a (type, "text");
+
+		g_object_unref (info);
 	}
 
 	return is_text;
@@ -935,21 +908,29 @@ uri_is_text (gchar *uri)
 static gboolean
 check_file_size (gchar *uri)
 {
-	GnomeVFSResult result;
-	GnomeVFSFileInfo *info;
+	GFile *file;
+	GFileInfo *info;
+	GError *error = NULL;
 	gboolean file_size_ok = FALSE;
 
-	info = gnome_vfs_file_info_new ();
-	result = gnome_vfs_get_file_info (uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+	file = g_file_new_for_uri (uri);
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SIZE,
+				  0, NULL, &error);
+	g_object_unref (file);
 
-	if (result == GNOME_VFS_OK) {
-		if (info->size <= DROP_FILE_PASTE_MAX_SIZE)
-			file_size_ok = TRUE;
+	if (!info) {
+		g_printerr (_("Error retrieving file information for \"%s\": %s\n"),
+			      uri, error->message);
+		g_error_free (error);
 	} else {
-		g_printerr (_("Error retrieving file information for \"%s\": %s\n"), uri, gnome_vfs_result_to_string (result));
+		goffset size = g_file_info_get_size (info);
+
+		if (size <= DROP_FILE_PASTE_MAX_SIZE)
+			file_size_ok = TRUE;
+
+		g_object_unref (info);
 	}
 
-	gnome_vfs_file_info_unref (info);
 	return file_size_ok;
 }
 
@@ -978,8 +959,8 @@ void
 conversation_panel_update_colors (ConversationPanel *panel)
 {
 	palette_alloc (panel->priv->xtext);
-	xtext_set_palette (XTEXT (panel->priv->xtext), colors);
-	xtext_refresh     (XTEXT (panel->priv->xtext), FALSE);
+	gtk_xtext_set_palette (GTK_XTEXT (panel->priv->xtext), colors);
+	gtk_xtext_refresh     (GTK_XTEXT (panel->priv->xtext), FALSE);
 }
 
 void
@@ -989,10 +970,10 @@ conversation_panel_add_session (ConversationPanel *panel, struct session *sess,
 	xtext_buffer *buffer;
 	gint          notify;
 
-	buffer = xtext_buffer_new (XTEXT (panel->priv->xtext));
+	buffer = gtk_xtext_buffer_new (GTK_XTEXT (panel->priv->xtext));
 
 	client = gconf_client_get_default ();
-	xtext_set_time_stamp (buffer, gconf_client_get_bool (client, "/apps/xchat/irc/showtimestamps", NULL));
+	gtk_xtext_set_time_stamp (buffer, gconf_client_get_bool (client, "/apps/xchat/irc/showtimestamps", NULL));
 	notify = gconf_client_notify_add (client, "/apps/xchat/irc/showtimestamps",
 	                                  (GConfClientNotifyFunc) timestamps_changed, buffer, NULL, NULL);
 	g_object_unref (client);
@@ -1013,12 +994,15 @@ conversation_panel_set_current (ConversationPanel *panel, struct session *sess)
 	panel->priv->current = sess;
 	if (sess) {
 		buffer = g_hash_table_lookup (panel->priv->buffers, sess);
-		xtext_buffer_show (XTEXT (panel->priv->xtext), buffer, TRUE);
+		gtk_xtext_buffer_show (GTK_XTEXT (panel->priv->xtext), buffer, TRUE);
 	} else {
-		xtext_buffer_show (XTEXT (panel->priv->xtext), XTEXT (panel->priv->xtext)->orig_buffer, TRUE);
+		gtk_xtext_buffer_show (GTK_XTEXT (panel->priv->xtext), GTK_XTEXT (panel->priv->xtext)->orig_buffer, TRUE);
 	}
 
-	conversation_panel_remove_tooltip (panel);
+	gtk_widget_set_has_tooltip(GTK_WIDGET(panel), FALSE);
+	gtk_widget_set_has_tooltip(GTK_WIDGET(panel), TRUE);
+	g_free(panel->priv->tooltip_nick);
+	panel->priv->tooltip_nick = NULL;
 }
 
 void
@@ -1065,7 +1049,7 @@ conversation_panel_save_current (ConversationPanel *panel)
 			g_error_free (error);
 		} else {
 			gint fd = g_io_channel_unix_get_fd (file);
-			xtext_save (XTEXT (panel->priv->xtext), fd);
+			gtk_xtext_save (GTK_XTEXT (panel->priv->xtext), fd);
 			g_io_channel_shutdown (file, TRUE, &error);
 
 			if (error) {
@@ -1086,16 +1070,13 @@ conversation_panel_clear (ConversationPanel *panel, struct session *sess)
 	xtext_buffer *buffer;
 
 	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
-	xtext_clear (buffer);
-	xtext_refresh (XTEXT (panel->priv->xtext), FALSE);
+	gtk_xtext_clear (buffer);
+	gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), FALSE);
 }
 
 static void
-conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, guchar *text, int len, gboolean indent)
+conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, char *text, int len, gboolean indent)
 {
-	int            leftlen;
-	unsigned char *tab;
-
 	if (len == 0) {
 		return;
 	}
@@ -1110,35 +1091,37 @@ conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, g
 		memcpy (new_text, stamp, stamp_size);
 		g_free (stamp);
 		memcpy (new_text + stamp_size, text, len);
-		xtext_append (buffer, new_text, len + stamp_size);
+		gtk_xtext_append (buffer, new_text, len + stamp_size);
 		g_free (new_text);
 		return;
 	}
 
-	tab = strchr (text, '\t');
+	char *tab = strchr (text, '\t');
 	if (tab && tab < (text + len)) {
-		leftlen = tab - text;
+		int leftlen = tab - text;
 
 		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0) {
 			text = tab+1;
 			len -= leftlen;
-			xtext_append_indent (buffer, 0, 0, text, len);
+			gtk_xtext_append_indent (buffer, 0, 0, (unsigned char*) text, len, time(NULL));
 		} else {
 			strncpy (buffer->laststamp, text, leftlen);
 			buffer->laststamp[leftlen]=0;
-			xtext_append_indent (buffer, text, leftlen, tab + 1, strlen (text) - leftlen - 1);
+			gtk_xtext_append_indent (buffer, (unsigned char*) text, leftlen,
+			                         (unsigned char*) tab + 1, strlen (text) - leftlen - 1,
+			                         time(NULL));
 		}
 	} else {
-		xtext_append_indent (buffer, 0, 0, text, len);
+		gtk_xtext_append_indent (buffer, 0, 0, (unsigned char*) text, len, time(NULL));
 	}
 }
 
 void
-conversation_panel_print (ConversationPanel *panel, struct session *sess, guchar *text, gboolean indent)
+conversation_panel_print (ConversationPanel *panel, struct session *sess, char *text, gboolean indent)
 {
 	xtext_buffer *buffer;
-	guchar       *last_text = text;
-	int           len       = 0;
+	char *last_text = text;
+	int len = 0;
 
 	if (strlen (text) == 0) {
 		return;
@@ -1189,9 +1172,9 @@ conversation_panel_remove_session (ConversationPanel *panel, struct session *ses
 }
 
 static void
-conversation_panel_lastlog_foreach (XText *xtext, guchar *text, fe_lastlog_info *info)
+conversation_panel_lastlog_foreach (GtkXText *xtext, char *text, fe_lastlog_info *info)
 {
-	if (nocasestrstr (text, info->sstr)) {
+	if (nocasestrstr (text, (char*) info->sstr)) {
 		conversation_panel_print (info->panel, info->sess, text, prefs.indent_nicks);
 	}
 }
@@ -1204,29 +1187,29 @@ conversation_panel_lastlog (ConversationPanel *panel, struct session *sess, stru
 	buffer  = g_hash_table_lookup (panel->priv->buffers, sess);
 	lbuffer = g_hash_table_lookup (panel->priv->buffers, lsess);
 
-	if (xtext_is_empty (buffer)) {
+	if (gtk_xtext_is_empty (buffer)) {
 		conversation_panel_print (panel, lsess, _("Search buffer is empty.\n"), TRUE);
 	} else {
 		fe_lastlog_info info;
 		info.panel = panel;
 		info.sess  = lsess;
-		info.sstr  = sstr;
+		info.sstr  = (unsigned char*) sstr;
 
-		xtext_foreach (buffer, (XTextForeach) conversation_panel_lastlog_foreach, &info);
+		gtk_xtext_foreach (buffer, (GtkXTextForeach) conversation_panel_lastlog_foreach, &info);
 	}
 }
 
 void
 conversation_panel_clear_selection (ConversationPanel *panel)
 {
-	xtext_selection_clear_full (XTEXT (panel->priv->xtext)->buffer);
-        xtext_refresh (XTEXT (panel->priv->xtext), TRUE);
+	gtk_xtext_selection_clear_full (GTK_XTEXT (panel->priv->xtext)->buffer);
+        gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), TRUE);
 }
 
 gpointer
 conversation_panel_search (ConversationPanel *panel, const gchar *text, gpointer start, gboolean casem, gboolean reverse)
 {
-	return xtext_search (XTEXT (panel->priv->xtext), text, start, casem, reverse);
+	return gtk_xtext_search (GTK_XTEXT (panel->priv->xtext), text, start, casem, reverse);
 }
 
 void
@@ -1235,7 +1218,7 @@ conversation_panel_page_up (ConversationPanel *panel)
 	GtkAdjustment *adj;
 	int end, value;
 
-	adj = xtext_get_adjustment (XTEXT (panel->priv->xtext));
+	adj = GTK_XTEXT(panel->priv->xtext)->adj;
 	end = adj->upper - adj->lower - adj->page_size;
 	value = adj->value - (adj->page_size - 1);
 	if (value < 0) {
@@ -1253,7 +1236,7 @@ conversation_panel_page_down (ConversationPanel *panel)
 	GtkAdjustment *adj;
 	int value, end;
 
-	adj = xtext_get_adjustment (XTEXT (panel->priv->xtext));
+	adj = GTK_XTEXT(panel->priv->xtext)->adj;
 	end = adj->upper - adj->lower - adj->page_size;
 	value = adj->value + (adj->page_size - 1);
 	if (value < 0) {
@@ -1268,13 +1251,13 @@ conversation_panel_page_down (ConversationPanel *panel)
 void
 conversation_panel_copy_selection (ConversationPanel *panel)
 {
-	xtext_copy_selection (XTEXT (panel->priv->xtext));
+	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext));
 }
 
 void
 conversation_panel_queue_tdraw (ConversationPanel *panel)
 {
-	if (XTEXT (panel->priv->xtext)->transparent) {
+	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
 		gtk_widget_queue_draw (panel->priv->xtext);
 	}
 }
@@ -1282,8 +1265,8 @@ conversation_panel_queue_tdraw (ConversationPanel *panel)
 static void
 redraw_transparency (ConversationPanel *panel)
 {
-	if (XTEXT (panel->priv->xtext)->transparent) {
-		xtext_refresh (XTEXT (panel->priv->xtext), 1);
+	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
+		gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), 1);
 	}
 }
 
@@ -1309,11 +1292,11 @@ root_event_cb (GdkXEvent *xev, GdkEventProperty *event, ConversationPanel *panel
 void
 conversation_panel_check_marker_visibility (ConversationPanel *panel)
 {
-	xtext_check_marker_visibility (XTEXT (panel->priv->xtext));
+	gtk_xtext_check_marker_visibility (GTK_XTEXT (panel->priv->xtext));
 }
 
 void
 conversation_panel_set_show_marker (ConversationPanel *panel, gboolean show_marker)
 {
-	xtext_set_show_marker (XTEXT (panel->priv->xtext), show_marker);
+	gtk_xtext_set_show_marker (GTK_XTEXT (panel->priv->xtext), show_marker);
 }
